/*
** Module   : EditorConfig.java
** Abstract : stores Editor component state
**
** Copyright (c) 2006-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ---------------------------Description----------------------------
** 001 SIY 20060428   @26085 Created initial version.
** 002 GES 20060803   @28405 Made externalizable for better performance. Coding std cleanups.
** 003 SIY 20060810   @28645 Moved code which validates and adjusts configuration variables 
**                           from Editor.java.
** 004 SIY 20100619          Switching to own implementation of character UI.
** 005 SIY 20100804          Changes in imports, some refactorings.
** 006 SIY 20111101          Moved from CHUI package. Minor cleanups/refactorings in preparation 
**                           for GUI client.
** 007 HC  20140911          GUI coordinate support - data types refactoring.
** 008 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 009 HC  20141021          Implemented window move/position support.
** 010 GES 20141124          Javadoc change.
** 011 GES 20150103          Moved state here from the client class to enable cursor-line and
**                           cursor-char attributes to be read/written on the server side. These
**                           are now calculated values since they don't directly correspond to
**                           the client-side state.
** 012 IAS 20150325          Added returnInserted attribute and selectionActive flag
*/

package com.goldencode.p2j.ui;

import java.io.*;

/**
 * The configuration container class which holds attributes specific for the Editor widget.
 */
public class EditorConfig
extends ControlTextConfig
{
   /** BUFFER-CHARS Attribute */
   public int bufferChars = -1;
   
   /** BUFFER-LINES Attribute */
   public int bufferLines = -1;
   
   /** CURSOR-OFFSET Attribute */
   public int cursorOffset = -1;
   
   /** Cursor X position relative to left edge of the inner rectangle of widget (0-based). */
   public int cursorX = 0;
   
   /** Cursor Y position relative to to edge of the inner rectangle of widget (0-based). */
   public int cursorY = 0;
   
   /** Horizontal scrolling index of the text (0-based). */
   public int scrollX = 0;
   
   /** Vertical scrolling index of the text (0-based). */
   public int scrollY = 0;

   /** INNER-CHARS Attribute */
   public int innerChars = -1;
   
   /** The number of text lines in the widget. */
   public int innerLines = -1;
   
   /** LENGTH Attribute */
   public int length = 0;
   
   /** MAX-CHARS Attribute */
   public int maxChars = -1;
   
   /** NUM-LINES Attribute */
   public int numLines = 0;
   
   /** NUM-REPLACED Attribute */
   public int numReplaced = 0;
  
   /** selection is active flag */ 
   public boolean selectionActive = false;

   /** SELECTION-END Attribute */
   public int selectionEnd = -1;
   
   /** SELECTION-START Attribute */
   public int selectionStart = -1;
   
   /** AUTO-INDENT Attribute */
   public boolean autoIndent = false;
   
   /** EMPTY Attribute */
   public boolean empty = false;
   
   /** SCROLLBAR-HORIZONTAL Attribute */
   public boolean scrollbarHorizontal = false;
   
   /** SCROLLBAR-VERTICAL Attribute */
   public boolean scrollbarVertical = false;
   
   /** TEXT-SELECTED Attribute */
   public boolean textSelected = false;
   
   /** WORD-WRAP Attribute */
   public boolean wordWrap = true;
   
   /** BOX attribute */
   public boolean box = true;

   /** SELECTION-TEXT Attribute */
   public String selectionText = null;
   
   /** RETURN-INSERTED attribute */
   public boolean returnInserted = false;
   
   /** TABLE Attribute */
   public String table = null;
   
   /**
    * Default constructor (only used in deserialization).
    */
   public EditorConfig()
   {
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected EditorConfig(int id)
   {
      super(id);
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected EditorConfig(WidgetId id)
   {
      super(id);
   }

   /**
    * Synchronize the configuration with the widget state.
    */
   @Override
   public void syncWithWidget()
   {
      super.syncWithWidget();
      
      if (onClientSide())
      {
         // nothing for the client-side
         return;
      }
      
      // if no scroll bars are defined, then reset is-box flag
      if (this.scrollbarVertical || this.scrollbarHorizontal)
         this.box = true;
      else
         this.box = false;

      int delta = this.box ? 2:0;

      // safe, comparison of scaled doubles
      if (this.widthChars == 0 && this.heightChars == 0)
      {
         // adjust chars/lines by inner chars/lines
         this.widthChars = Coordinate.scale(this.innerChars + delta);
         this.heightChars = Coordinate.scale(this.innerLines + delta);
      }
      else
      {
         // adjust inner chars/lines by chars/lines
         this.innerChars = (int) this.widthChars - delta;
         this.innerLines = (int) this.heightChars - delta;
      }
      
      // buffer-chars must be greater or equal to visible area width
      if (this.bufferChars < this.innerChars)
         this.bufferChars = this.innerChars;
      
      // buffer-lines, if specified, must be at least equal to visible area height 
      if (this.bufferLines > 0 && this.bufferLines < this.innerLines)
         this.bufferLines = this.innerLines;
   }

   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      EditorConfig cfg = (EditorConfig) config;
      
      bufferChars = cfg.bufferChars;
      bufferLines = cfg.bufferLines;
      cursorX = cfg.cursorX;
      cursorY = cfg.cursorY;
      scrollX = cfg.scrollX;
      scrollY = cfg.scrollY;
      cursorOffset = cfg.cursorOffset;
      innerChars = cfg.innerChars;
      innerLines = cfg.innerLines;
      length = cfg.length;
      maxChars = cfg.maxChars;
      numLines = cfg.numLines;
      numReplaced = cfg.numReplaced;
      selectionActive = cfg.selectionActive;
      selectionEnd = cfg.selectionEnd;
      selectionStart = cfg.selectionStart;
      autoIndent = cfg.autoIndent;
      empty = cfg.empty;
      scrollbarHorizontal = cfg.scrollbarHorizontal;
      scrollbarVertical = cfg.scrollbarVertical;
      textSelected = cfg.textSelected;
      wordWrap = cfg.wordWrap;
      box = cfg.box;
      selectionText = cfg.selectionText;
      table = cfg.table;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      bufferChars         = in.readInt();
      bufferLines         = in.readInt();
      cursorX             = in.readInt();
      cursorY             = in.readInt();
      scrollX             = in.readInt();
      scrollY             = in.readInt();
      cursorOffset        = in.readInt();
      innerChars          = in.readInt();
      innerLines          = in.readInt();
      length              = in.readInt();
      maxChars            = in.readInt();
      numLines            = in.readInt();
      numReplaced         = in.readInt();
      selectionActive     = in.readBoolean();
      selectionEnd        = in.readInt();
      selectionStart      = in.readInt();
      autoIndent          = in.readBoolean();
      empty               = in.readBoolean();
      scrollbarHorizontal = in.readBoolean();
      scrollbarVertical   = in.readBoolean();
      textSelected        = in.readBoolean();
      wordWrap            = in.readBoolean();
      box                 = in.readBoolean();
      selectionText       = (String) in.readObject();
      table               = (String) in.readObject();
      returnInserted      = in.readBoolean();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeInt(bufferChars);
      out.writeInt(bufferLines);
      out.writeInt(cursorX);
      out.writeInt(cursorY);
      out.writeInt(scrollX);
      out.writeInt(scrollY);
      out.writeInt(cursorOffset);
      out.writeInt(innerChars);
      out.writeInt(innerLines);
      out.writeInt(length);
      out.writeInt(maxChars);
      out.writeInt(numLines);
      out.writeInt(numReplaced);
      out.writeBoolean(selectionActive);
      out.writeInt(selectionEnd);
      out.writeInt(selectionStart);
      out.writeBoolean(autoIndent);
      out.writeBoolean(empty);
      out.writeBoolean(scrollbarHorizontal);
      out.writeBoolean(scrollbarVertical);
      out.writeBoolean(textSelected);
      out.writeBoolean(wordWrap);
      out.writeBoolean(box);
      out.writeObject(selectionText);
      out.writeObject(table);
      out.writeBoolean(returnInserted);
   }
}
